package at.ac.uibk.gitsearch.security.oauth2;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.security.authentication.event.AbstractAuthenticationEvent;
import org.springframework.security.authentication.event.AuthenticationSuccessEvent;
import org.springframework.security.authentication.event.InteractiveAuthenticationSuccessEvent;
import org.springframework.security.oauth2.client.authentication.OAuth2LoginAuthenticationToken;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.stereotype.Service;

import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.security.AuthoritiesConstants;
import at.ac.uibk.gitsearch.service.UserService;
import at.ac.uibk.gitsearch.service.dto.UserDTO;
import at.ac.uibk.gitsearch.service.mapper.UserMapper;

/**
 * inserts a new authenticated user into Database
 * @author Michael Breu
 *
 */
@Service
public class OAuth2AuthenticationSuccessEventHandler{

	private final Set<String> defaultAuthorities = new HashSet<String>();

	@Autowired
	private UserDetailsFetcher userDetailsFetcher;
	@Autowired
	private UserMapper userMapper;
	
	public OAuth2AuthenticationSuccessEventHandler() {
		defaultAuthorities.add(AuthoritiesConstants.USER);
	}
    @Autowired
    private UserService userService;

    @EventListener({AuthenticationSuccessEvent.class, InteractiveAuthenticationSuccessEvent.class})
    public void processAuthenticationSuccessEvent(AbstractAuthenticationEvent  e) {

    	if (!(e.getAuthentication() instanceof OAuth2LoginAuthenticationToken)) {
    		return;
    	}
        Object principal = e.getAuthentication().getPrincipal();
        
        if (principal instanceof OidcUser) {
        	OidcUser oidcUser = (OidcUser) principal;
        	String email = oidcUser.getEmail();
			
	        Optional<User> userO = userService.getUserWithAuthoritiesByEmail(email);
	        
	        if(userO.isPresent()) {
	        	UserDTO userDto = userMapper.userToUserDTO(userO.get());
	        	boolean hasChanged = userDetailsFetcher.updateUserDetails(userDto, oidcUser, (OAuth2LoginAuthenticationToken) e.getAuthentication());
	        	if (hasChanged)
	        		userService.updateUser(userDto);
	        } else {
	        	UserDTO u = new UserDTO();
	        	u.setFirstName(oidcUser.getGivenName());
	        	u.setLastName(oidcUser.getFamilyName());
	        	u.setLogin(email);
	        	u.setActivated(true);
	        	u.setLastModifiedBy("system");
	        	u.setCreatedDate(java.time.Instant.now());
	        	u.setAuthorities(defaultAuthorities);
	        	u.setEmail(oidcUser.getEmail());
	        	@SuppressWarnings("unused")
				boolean hasChanged = userDetailsFetcher.updateUserDetails(u, oidcUser, (OAuth2LoginAuthenticationToken) e.getAuthentication());
	        	userService.createUser(u);
	        }

			
		}
    }   
}