package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.es.model.DocumentInfo;
import at.ac.uibk.gitsearch.repository.search.GitFilesRepository;
import at.ac.uibk.gitsearch.service.dto.GitFilesAggregationDTO;
import at.ac.uibk.gitsearch.service.dto.GitFilesPageDetailsDTO;
import at.ac.uibk.gitsearch.service.dto.SearchInputDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.util.List;

/**
 * REST controller for managing {@link DocumentInfo}.
 */
@RestController
@RequestMapping("/api")
@Transactional
public class GitFilesResource {

    private final Logger log = LoggerFactory.getLogger(GitFilesResource.class);

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final GitFilesRepository gitFilesRepository;

    public GitFilesResource(GitFilesRepository gitFilesRepository) {
        this.gitFilesRepository = gitFilesRepository;
    }

    /**
     * //     * {@code SEARCH  /_search/git-files/page-details?query=:query} : search for the searchResult corresponding
     * //     * to the query.
     * //     *
     * //     * @param query the query of the searchResult search.
     * //     * @return the result of the search.
     * //
     */
    @GetMapping("/_search/git-files/page-details")
    public GitFilesPageDetailsDTO
    gitFilesPageDetails(@RequestParam String fulltextQuery, @RequestParam String metadataProgrammingLanguage,
                        @RequestParam String metadataKeywords, @RequestParam String metadataNaturalLanguage,
                        @RequestParam String metadataLicense, @RequestParam String metadataAuthor,
                        @RequestParam List<String> selectedRepository,
                        @RequestParam List<String> selectedUniversity, @RequestParam List<String> selectedFileFormat,
                        @RequestParam int page, @RequestParam int pageSize) throws IOException {
        SearchInputDTO searchInput = new SearchInputDTO(fulltextQuery, metadataProgrammingLanguage, metadataKeywords,
            metadataNaturalLanguage, metadataLicense, metadataAuthor, selectedRepository, selectedUniversity,
            selectedFileFormat, page);
        log.debug("REST request to search gitFilesPageDetails for searchInput {}, pageSize {}", searchInput, pageSize);
        // return searchResultSearchRepository.findByContent(query);
        return gitFilesRepository.pageDetails(searchInput, pageSize);
    }

    /**
     * //     * {@code SEARCH  /_search/git-files/aggregation?query=:query} : search for the searchResult corresponding
     * //     * to the query.
     * //     *
     * //     * @param query the query of the searchResult search.
     * //     * @return the result of the search.
     * //
     */
    @GetMapping("/_search/git-files/aggregation")
    public GitFilesAggregationDTO gitFilesAggregation(@RequestParam String query) throws IOException {
        log.debug("REST request to search gitFilesAggregation for query {}", query);
        // return searchResultSearchRepository.findByContent(query);
        return gitFilesRepository.aggregation(query);
    }
}
